/*********************************************************************
 * [SliderPuzzle スクリプト] PieceSizeInfo.js
 * 一枚絵の画像の分割数が指定されなかった場合に、
 * このクラスを用いて適当に分割数を求める。
 *
 * [使い方]
 *  幅432px高さ374pxという情報に対し、分割数を求める：
 *    var info = new PieceSizeInfo();
 *    info.calc(432, 374);
 *    var rowCount = info.getOptimalRow().count;
 *    var colCount = info.getOptimalCol().count;
 *  結果は縦横とも6分割（ピースのサイズはW62xH72）。
 *  ただし横方向は割り切れていないため、パズル完成後の画像は右が少し切れる。
 *
 * Copyright (C) 2015 ASANO Sei All Rights Reserved.
 *********************************************************************/
/*********************************************************************
 * [PieceSizeInfoItem]
 * 個々のサイズ情報を保持するクラス
 *
 * [引数] なし
 * [戻り] なし
 *********************************************************************/
function PieceSizeInfoItem() {
	this.yo   = 0; //剰余
	this.count= 0; //分割数
	this.size = 0; //サイズ(px)
	this.gap  = 0; //基準サイズとの差
}
/*********************************************************************
 * [PieceSizeInfo]
 * 算出したサイズ情報を保持するクラス
 *********************************************************************/
// 分割許容範囲
var PieceSizeInfo_range = {
	count : { min:  3, max: 10 } //分割数の範囲
	,size : { min: 50, max: 100, standard: 75 } //分割後の画像サイズ(px)の範囲
}
/*-------------------------------------------------------------------*
 * コンストラクタ
 *
 * [引数] なし
 * [戻り] なし
 *-------------------------------------------------------------------*/
function PieceSizeInfo() {
	this.range = PieceSizeInfo_range;
	this.checkRange = PieceSizeInfo_checkRange;
	this.calc = PieceSizeInfo_calc;
	this.calcSub = PieceSizeInfo_calcSub;
	this.getOptimalCount = PieceSizeInfo_getOptimalCount;
	this.getOptimalRow = PieceSizeInfo_getOptimalRow;
	this.getOptimalCol = PieceSizeInfo_getOptimalCol;
	this.sortFunction = PieceSizeInfo_sortFunction;
	this.sort = PieceSizeInfo_sort;
	this.colItems = null;
	this.rowItems = null;
}
/*-------------------------------------------------------------------*
 * ソート用比較関数
 *
 * [引数] item1:
 *        item2:
 * [戻り] 順序
 *-------------------------------------------------------------------*/
function PieceSizeInfo_sortFunction(item1, item2) {
	return item1.gap - item2.gap;
}
/*-------------------------------------------------------------------*
 * 指定した値が分割許容範囲に含まれているかを返却する。
 *
 * [引数] val: 値
 * [戻り] 分割許容範囲に含まれいればtrue
 *-------------------------------------------------------------------*/
function PieceSizeInfo_checkRange(val) {
	if (val < this.range.size.min) return false;
	if (val > this.range.size.max) return false;
	return true;
}
/*-------------------------------------------------------------------*
 * 指定した幅と高さに対し、それぞれ分割数の候補配列を作成する。
 *
 * [引数] width : 幅
 *        height: 高さ
 * [戻り] 分割数の候補配列
 *-------------------------------------------------------------------*/
function PieceSizeInfo_calc(width, height) {
	this.colItems = this.calcSub(width);
	this.sort(this.colItems);

	this.rowItems = this.calcSub(height);
	this.sort(this.rowItems);
}
function PieceSizeInfo_calcSub(val) {
	var array = [];
	for (var i=this.range.count.max ; i>=this.range.count.min ; i--) {
		var yo = val % i;
		var num = parseInt(val / i);
		if (!this.checkRange(num)) continue;
		var gap = this.range.size.standard - num;
		if (gap < 0) gap *= -1;

		var item = new PieceSizeInfoItem();
		item.yo    = yo;
		item.count = i;
		item.size  = num;
		item.gap   = gap;
		if (!array[yo]) array[yo] = [];
		array[yo].push(item); //剰余を添え字にして、候補オブジェクトを追加
	}
	return array;
}
/*-------------------------------------------------------------------*
 * 剰余ごとの候補配列を、基準値との差で昇順ソートする。
 *
 * [引数] array: colItemsまたはrowItems
 * [戻り] なし
 *-------------------------------------------------------------------*/
function PieceSizeInfo_sort(array) {
	for (var i=0 ; i<array.length ; i++) {
		if (!array[i]) continue;
		array[i].sort(this.sortFunction);
	}
}
/*-------------------------------------------------------------------*
 * 縦の分割数の候補配列のうち最適解を返却する。
 *-------------------------------------------------------------------*/
function PieceSizeInfo_getOptimalRow() {
	return this.getOptimalCount(this.rowItems);
}
/*-------------------------------------------------------------------*
 * 横の分割数の候補配列のうち最適解を返却する。
 *-------------------------------------------------------------------*/
function PieceSizeInfo_getOptimalCol() {
	return this.getOptimalCount(this.colItems);
}
/*-------------------------------------------------------------------*
 * 分割数の候補配列から、以下の条件を満たす要素を最適解として返却する。
 *  (1) 剰余が最小
 *  (2) (1)の中で基準値(range.size.standart)との差が最小
 *
 * [引数] array : colItemsまたはrowItems
 * [戻り] 最適解。なければnull
 *-------------------------------------------------------------------*/
function PieceSizeInfo_getOptimalCount(array) {
	var sizeItems = null;
	for (var i=0 ; i<array.length ; i++) {
		sizeItems = array[i];
		if (!sizeItems) continue;
		//基準値との差で昇順ソート済なので、存在する候補配列の先頭要素を取得
		return sizeItems[0];
	}
	return null;
}


//************************************************
//テストコード
/*
function showMsg(msg) {
	WScript.Echo(msg);
}
function getProperties(obj) {
	if (!obj) return "";
	var result = [];
	for (var name in obj) {
		result.push(name + ":" + obj[name]);
	}
	return result.join(", ");
}
var dataArray = [
	 {width:198,height:200}
	,{width:500,height:550}
	,{width:402,height:380}
	,{width:432,height:374}
	,{width:4320,height:3740}
];
var info = new PieceSizeInfo();
var message = [];
for (var i=0 ; i<dataArray.length ; i++) {
	var item = dataArray[i];
	message.push("[" + i + "] W" + item.width + "xH" + item.height);
	info.calc(item.height, item.width);
	var rowItem = info.getOptimalRow();
	var colItem = info.getOptimalCol();
	message.push("\t[ROW] " + (rowItem ? getProperties(rowItem) : "none"));
	message.push("\t[COL] " + (colItem ? getProperties(colItem) : "none"));
}
showMsg(message.join("\n"));
*/
